#!/usr/bin/bash
if [[ $1 == runserver ]]; then
    # Prepare database, this does nothing to already existing and latest db versions,
    # 11/10 sysadmins would recommend doing this every time
    oraculum init_db
    oraculum upgrade_db

    gunicorn --workers 2 --bind 0.0.0.0:5001 oraculum
fi

if [[ $1 == runproxy ]]; then
    # Set the url to the gunicorn worker
    sed -i "s,proxy_pass http://0.0.0.0:5001;,proxy_pass http://${ORACULUM_SERVICE_HOST}:5001;,g" /etc/nginx/conf.d/oraculum.conf
    nginx
fi

if [[ $1 == runworker ]]; then
    celery -A oraculum.celery_app worker --concurrency=8
fi

if [[ $1 == runbeat ]]; then
    celery -A oraculum.celery_app worker -B -s /tmp/celerybeat-schedule
fi

if [[ $1 == debug ]]; then
    # Prepare development settings
    echo "SQLALCHEMY_DATABASE_URI = 'sqlite:////var/tmp/oraculum.sqlite'" >> /etc/oraculum/settings.py
    install -p -m 0644 /opt/app-root/src/oraculum/conf/client_secrets.json.example /etc/oraculum/client_secrets.json
    echo "OIDC_CLIENT_SECRETS = '/etc/oraculum/client_secrets.json'" >> /etc/oraculum/settings.py

    # Prepare database
    oraculum init_db
    oraculum upgrade_db

    # Start redis and gunicorn
    redis-server --daemonize yes
    gunicorn --workers 2 --daemon --bind 0.0.0.0:5001 oraculum

    # We want nginx to run in the background for debug run
    sed -i '/daemon/d' /etc/nginx/nginx.conf

    # We want debug level logging
    echo "DEBUG = True" >> /etc/oraculum/settings.py

    # And finally fire up nginx and beat
    nginx
    celery -A oraculum.celery_app worker -B
fi
