import pytest
import mock

from oraculum.utils import orphans

class TestOrphans(object):


    def test_orphans(self):
        """
        * notes packages being orphaned
        A -- B -- C*
                -- D* -- E*
                -- E*
            -- F*
        """
        DATA = {
                "affected_packages": {
                    'A': ['B', 'F'],
                    'B': ['C', 'D', 'E'],
                    'C': [],
                    'D': ['E'],
                    'E': [],
                    'F': []
                },
                "status_change": {
                    'C': '2020-01-01T00:00:00',
                    'D': '2020-01-01T00:00:01',
                    'E': '2020-01-01T00:00:02',
                    'F': '2020-01-01T00:00:03',

                }
        }
        out = orphans.get_orphans(["A", "F", "Z"], DATA)
        # Not affected
        o_z = {
            "orphaned": False,
            "depends_on_orphaned": False,
            "direct_dependencies": [],
            "remote_dependencies": [],
            "problematic_since": None,
            "dot_graph": "",
            "vis_js": {"nodes": [], "edges": []}
            }
        assert out["Z"] == o_z

        # Directly orphaned
        o_f = {
            "orphaned": True,
            "depends_on_orphaned": False,
            "direct_dependencies": [],
            "remote_dependencies": [],
            "problematic_since": '2020-01-01T00:00:03',
            "dot_graph": "",
            "vis_js": {"nodes": [], "edges": []}
            }
        assert out["F"] == o_f

        # Complete example
        o_a = {
            "orphaned": False,
            "depends_on_orphaned": True,
            "direct_dependencies": ['F'],
            "remote_dependencies": ['C', 'D', 'E'],
            "problematic_since": '2020-01-01T00:00:00'
            }
        dg = sorted(out["A"]["dot_graph"].split('\n'))
        vis_js = out["A"]["vis_js"]
        del(out["A"]["dot_graph"])
        del(out["A"]["vis_js"])

        out["A"]["remote_dependencies"].sort()
        assert out["A"] == o_a

        o_dg = sorted([
            '"A" -- "B" -- "C";',
            '"A" -- "B" -- "D";',
            '"A" -- "B" -- "E";',
            '"A" -- "F";'
            ])
        assert o_dg == dg

        assert len(vis_js.keys()) == 2
        assert 'nodes' in vis_js.keys()
        assert 'edges' in vis_js.keys()

        nodes = sorted([(n['id'], n['label']) for n in vis_js['nodes']])
        assert nodes == [(0, 'A'), (1, 'B'), (2, 'C'), (3, 'D'), (4, 'E'), (5, 'F')]

        edges = sorted([(e['from'], e['to']) for e in vis_js['edges']])
        assert edges == [(0, 1), (0, 5), (1, 2), (1, 3), (1, 4)]
